
;****************************************************************************
;		File Name:		FOTBALEK
;		Date:			Kvten 2011
;		File Version:	
;		HW: 			board "FOTBAL_II"
;		Author:			VC - Allcomp
;
;****************************************************************************
;		Description: 	Ridici jednotka pro tabuli fotbal
;					
;		 Port : 	A0 - Temperature		B0 - A0 (10MIN)
;			A1 - Svetlo			B1 - A1 (1MIN)
;			A2 - TL-MIN			B2 - A2 (10SEC)
;			A3 - TL-SEC			B3 - A3 (1SEC)
;			A4 - TL-DOM			B4 - A4 (10DOM)
;			A5 - TL-HOS			B5 - A5 (1DOM)
;			A6 - X2 4000 kHz		B6 - A6 (10HOS)
;			A7 - X2				B7 - A7 (1HOS)
;
;		 Port : 	C0 - (Terc A) +DO		D0 - seg D
;			C1 - (Terc B) +DO		D1 - seg C
;			C2 - PWM			D2 - seg P
;			C3 - SCL				D3 - seg E
;			C4 - SDI (pistole)	 +DO		D4 - seg B
;			C5 - SDO			D5 - seg A
;			C6 - Tx				D6 - seg F
;			C7 - Rx				D7 - seg G	
;
;			E0 - enkoder SW			E2 - enkoder A
;			E1 - enkoder B			E3 - MCLR
;===========================================================================
;	V3.00	- 	TMR2IF= 1ms
;****************************************************************************

	list		p=16f887	; Typ mikrokontroleru
	#include	<p16f887.inc>	; Definice SFR registru a bitu 
			

	errorlevel	-302		; Zakazat hlaseni"Register..bank 0.."
			
;	****************************************************************************

SAE		EQU	0x01		;nastaveni segmentu EXTERNIHO displeje
SBE		EQU	0x80
SCE		EQU	0x40
SDE		EQU	0x20
SEE		EQU	0x08
SFE		EQU	0x04
SGE		EQU	0x02
SPE		EQU	0x10

SW_PORT	EQU	PORTA

ENKO_P		EQU	PORTE		;
A_ENKO		EQU	0x2
B_ENKO		EQU	0x1
S_ENKO		EQU	0x0

ANODY		EQU	PORTB		;nastaveni adresy anod displeje
AN0		EQU	0x0
AN1		EQU	0x1		;nastaveni jednotlivych anod
AN2		EQU	0x2
AN3		EQU	0x3
AN4		EQU	0x4
AN5		EQU	0x5
AN6		EQU	0x6
AN7		EQU	0x7

KATODY		EQU	PORTD		;0x008 nastaveni katod displeje
SA		EQU	0x20		;nastaveni segmentu displeje
SB		EQU	0x10
SC		EQU	0x02
SD		EQU	0x01
SE		EQU	0x08
SF		EQU	0x40
SG		EQU	0x80
SP		EQU	0x04

;	REGISTER KEYS:
TL_R		EQU	0x0
TL_L		EQU	0x1
TL_S		EQU	0x2
TL_LONG	EQU	0x3
TL_MIN		EQU	0x4
TL_SEC		EQU	0x5
TL_DOM		EQU	0x6
TL_HOS		EQU	0x7

;	REGISTER SWITCH:
XOR_AB		EQU	0x7
SW_MIN		EQU	0x2
SW_SEC		EQU	0x3
SW_DOM	EQU	0x4
SW_HOS		EQU	0x5

SERVIS_CODE		EQU	0x4346	;servisni kod "Hein" eken
SERVIS_CODE_ALL	EQU	0xA88C	;servisni kod ALLCOMP
POC_POK		EQU	0x03	;povoleny pocet pokusu
KANAL_MAX		EQU	0x08	;pocet kanalu (vrtulek MV02)maximum 8
TIME_OUTL		EQU	0x0A	;2.byte TIMER3 (10x)
TIME_OUT		EQU	0x78	;0xnn * 5 sec navrat do zakladniho MENU [0x78 ~ 10 min]
TIME_DISP		EQU	0x15	;0xnn * 100 ms prosviceni DONE, CODE
PULE			EQU	0x2D	;Polocas 45 min
TIM_LONG		EQU	0x80	;doba stiknuti TL_LONG

;verze hardware:	0=enkoder s potenciometrem + SMD 	1=enkoder ALPS
#define	HW	0			
;pouij Petrovu rutinu pro N kodr
#define	PETR_N	1

;Program Configuration Registers
  ifdef __DEBUG
	;Debug variant
	__CONFIG	_CONFIG1, _CP_OFF & _CPD_OFF & _DEBUG_ON & _LVP_OFF & _BOR_ON & _MCLRE_ON & _PWRTE_ON & _WDT_ON & _HS_OSC & _IESO_ON & _FCMEN_ON
	__CONFIG	_CONFIG2, _WRT_OFF & _BOR40V
  else
	;Release variant
	__CONFIG	_CONFIG1, _CP_ON & _CPD_OFF & _DEBUG_OFF & _LVP_OFF & _BOR_ON & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC & _IESO_ON & _FCMEN_ON
	__CONFIG	_CONFIG2, _WRT_OFF & _BOR40V
  endif


		org	0x000		; Addresa prvni instrukce
  ifdef __DEBUG
		nop			;pro krokovn
  endif
		goto 	START
		org	0x004		; vektor preruseni
			 	
;	*************    DEFINING PROGRAM VARIABLES   *************************			 	

W_TMP		EQU	0x70
STATUS_TMP	EQU	0x71
PCLATH_TMP	EQU	0x72
CSA1		EQU	0x73
CSA2		EQU	0x74		
CSA3		EQU	0x75
CSB1		EQU	0x76
CSB2		EQU	0x77
CSB3		EQU	0x78
CSC1		EQU	0x79	
CSC2		EQU	0x7A	
CSC3		EQU	0x7B		
;DAYM		EQU	0x7C	;counter DAY M	
;DAYH		EQU	0x7D	;counter DAY H	
MENU_TMP	EQU	0x7E	;pozice MENU
;KANAL		EQU	0x7F

	CBLOCK		0x20
	DIR_DISP	;oznaceni pozice MX displeje
	MIN		;minuty
	SEC		;sekundy
	DOM		;domaci
	HOS		;hoste
	STAV		;stav tlacitek MIN,SEC,HOS a DOM
	KEYS
	KEYS_LONG
	MENU
	POKUS
	ZAPAS		;prubeh zapasu 5-1.polocas,6-2.polocas,7-BLIK_M
	DESETINA	;desetina sekundy
	SETINA
	CISLO		;promenna pro prevod
	D100
	D10
	D1		;pro prevod na desitkovou soustavu
	TMP_L
	CISLA32
	CISLA10		;prevod desitkove
	DISP_D1		;display 10MIN
	DISP_D2		;1MIN
	DISP_D3		;10SEC
	DISP_D4		;1SEC
	DISP_D5		;10DOM
	DISP_D6		;1DOM
	DISP_D7		;10HOS
	DISP_D8		;1HOS
	POM		;pomocny registr
	POM2		;pomocny registr
	TIM		;pomocny registr casovacu
	RES_D		;pomocny registr (dlouhy stisk)
	SWITCH
	SWITCH_1
	TMP_W
	TIMR1
	TIMR2L
	TIMR2H		;casovac dlouhy
	TIMR3L
	TIMR3M
	TIMR3H
	T_LONG
	T_LONGM
	TXD0
	TXD1
	TXD2
	TXD3
	TXD4
	TXD5
	TXD6
	TXD7		;data pro externi display
	REF_D		;obnoveni dat externiho displeje
	JEDNA
	DESET
	STO
	TISIC
	DTISIC
	POLOCAS
	ENDC

;	===================  P O D P R O G R A M Y    cast 1   =========================

D_MENU	movfw	MENU
		andlw	B'00011111'	;omezeni skoku max 0x1F
		addwf	PCL,F
		goto	MENU_0		;0x00	
		goto	MENU_0		;0x01	
		goto	MENU_0		;0x02	
		goto	MENU_0		;0x03	
		goto	MENU_0		;0x04	
		goto	MENU_0		;0x05	
		goto	MENU_0		;0x06	
		goto	MENU_0		;0x07	
		goto	MENU_0		;0x08
		goto	MENU_09	;0x09 FOTBAL	

BCD_DISP	andlw	B'00011111'			;omezeni skoku max 0x1F
		addwf	PCL,F				;prevod BCD kod (scitaji se segmenty co nesviti)
		retlw	SG+SP				;0
		retlw	SA+SD+SE+SF+SG+SP		;1
		retlw	SC+SF+SP			;2
		retlw	SE+SF+SP			;3
		retlw	SA+SD+SE+SP			;4
		retlw	SB+SE+SP			;5
		retlw	SB+SP				;6
		retlw	SD+SE+SF+SG+SP		;7
		retlw	SP				;8
		retlw	SE+SP				;9
		retlw	SD+SP				;A
		retlw	SA+SB+SP			;b
		retlw	SB+SC+SG+SP			;C
		retlw	SA+SF+SP			;d
		retlw	SB+SC+SP			;E
		retlw	SB+SC+SD+SP			;F
		retlw	SA+SB+SC+SF+SP		;c	0x10
		retlw	SA+SB+SF+SP			;o	0x11
		retlw	SA+SB+SC+SD+SF+SP		;r	0x12
		retlw	SA+SB+SC+SD+SE+SF+SP	;-	0x13
		retlw	SA+SE+SP			;Y	0x14
		retlw	SA+SB+SC+SP			;t	0x15 
		retlw	SA+SB+SC+SG+SP		;L	0x16
		retlw	SA+SB+SF+SG+SP		;u	0x17
		retlw	SC+SD+SP			;P	0x18
		retlw	SA+SB+SD+SF+SP		;n	0x19
		retlw	SA+SB+SD+SG+SP		;li	0x1A
		retlw	SA+SD+SP			;H	0x1B
		retlw	SA+SD+SG+SP			;II	0x1C
		retlw	SA+SB+SC+SD+SG+SP		;l	0x1D
		retlw	0xFF				;	0x1E
		retlw	0xFF				;	0x1F
	

SEG7_EXT	andlw	B'00011111'				;omezeni skoku max 0x1F
		addwf	PCL,F					;prevod BCD kod (scitaji se segmenty co nesviti)
		retlw	SAE+SBE+SCE+SDE+SEE+SFE		;0
		retlw	SBE+SCE				;1
		retlw	SAE+SBE+SDE+SEE+SGE			;2
		retlw	SAE+SBE+SCE+SDE+SGE			;3
		retlw	SBE+SCE+SFE+SGE			;4
		retlw	SAE+SCE+SDE+SFE+SGE			;5
		retlw	SAE+SCE+SDE+SEE+SFE+SGE		;6
		retlw	SAE+SBE+SCE				;7
		retlw	SAE+SBE+SCE+SDE+SEE+SFE+SGE	;8
		retlw	SAE+SBE+SCE+SDE+SFE+SGE		;9
		retlw	SDE+SPE				;A  dopsat 3333333333
		retlw	SAE+SBE+SPE				;b
		retlw	SBE+SCE+SGE+SPE			;C
		retlw	SAE+SFE+SPE				;d
		retlw	SBE+SCE+SPE				;E
		retlw	SBE+SCE+SDE+SPE			;F
		retlw	SAE+SBE+SCE+SFE+SPE			;c	0x10
		retlw	SAE+SBE+SFE+SPE			;o	0x11
		retlw	SAE+SBE+SCE+SDE+SFE+SPE		;r	0x12
		retlw	SAE+SBE+SCE+SDE+SEE+SFE+SPE	;-	0x13
		retlw	SAE+SEE+SPE				;Y	0x14
		retlw	SAE+SBE+SCE+SPE			;t	0x15 
		retlw	SAE+SBE+SCE+SGE+SPE			;L	0x16
		retlw	SAE+SBE+SFE+SGE+SPE			;u	0x17
		retlw	SCE+SDE+SPE				;P	0x18
		retlw	SAE+SBE+SDE+SFE+SPE			;n	0x19
		retlw	SAE+SBE+SDE+SGE+SPE			;li	0x1A
		retlw	SAE+SDE+SPE				;H	0x1B
		retlw	SAE+SDE+SGE+SPE			;II	0x1C
		retlw	SAE+SBE+SCE+SDE+SGE+SPE		;l	0x1D
		retlw	0xFF					;	0x1E
		retlw	0xFF					;	0x1F	


		#include 	"Fotbal_300.inc"

;	--------------------------------------------------------------------------------------
;	------------------------------------------------------------------------------------------
;  STAV	| bit 7	| bit 6	| bit 5	| bit 4	| bit 3	| bit 2	| bit 1	| bit 0	|
;	------------------------------------------------------------------------------------------
;  	|-	|  -	| -	| -	| T_HOS | T_DOM	| T_SEC	| T_MIN	|
;	------------------------------------------------------------------------------------------ 
;	------------------------------------------------------------------------------------------
;  POM	| bit 7	| bit 6	| bit 5	| bit 4	| bit 3	| bit 2	| bit 1	| bit 0	|
;	------------------------------------------------------------------------------------------
;  	| 	| START	| Tl_L	| Tl_R	| XOR AB| T_TL3	| T_TLB	| T_TLA	|
;	------------------------------------------------------------------------------------------ 
;	------------------------------------------------------------------------------------------
; TIM	| bit 7	| bit 6	| bit 5	| bit 4	| bit 3	| bit 2	| bit 1	| bit 0	|
;	------------------------------------------------------------------------------------------
;Timer 	| BLIK	| LED	| HisL	| Req_L	| TlE>>	| Res>> |T1-30s	| T0-1s	|
;	------------------------------------------------------------------------------------------ 	

;	************************   HLAVNI PROGRAM   ********************************

START					; po spusteni
		bsf	STATUS,RP1
		bsf	STATUS,RP0	;>  BANK 3 <
		clrf	ANSEL		; Vynulovani ANSEL a ANSELH 
		clrf	ANSELH		; => vsechny vstupy digital (1=analog)
		bcf	STATUS,RP1	;>  BANK 1 <
		movlw	B'11000000'	;Leany B'10000000' (inverzni CLK)
		movwf	SSPSTAT	;SPI Master SDO, SCK
		movlw	0xF9		;0xFA (250)
		movwf	PR2		;preddelicka na 250
		movlw	B'10111000'
		movwf	OPTION_REG	; TIMER na PA4
		movlw	B'11111110'	;###
		movwf	TRISA	 	; Vsechny piny portu A vstup 
		movlw	B'00000000'	; B'11000000'
		movwf	TRISB		; piny 0-7 jako vystup 
		movlw	B'10000000' 
		movwf	TRISC		; Vsechny piny jako vystup krome C7
		clrf	TRISD		; Vsechny piny D jako vystup
		movlw	B'11111111' 
		movwf	TRISE		; Vsechny piny E jako vstup
		bcf	STATUS,RP0	;>  BANK 0 <
		movlw	B'00001100'
		movwf	CCP1CON	;PWM
		movlw	0x80		;
		movwf	CCPR1L		;PWM  50%
		movlw	B'00000101'
		movlw	B'00100010'
		movwf	SSPCON		;SPI on CKP, CLK Fosc/64
		movlw	B'00000101'
		movwf	T2CON		;aktivace TIMER2 ON,prescaler 1/4 /PR2 =1ms
		movlw	0xff
		movwf	DISP_D1		;zobrazeni na pozici A0
		movlw	0xff
		movwf	DISP_D2		;zobrazeni na pozici A1
		movlw	0xff
		movwf	DISP_D3		;zobrazeni na pozici A2
		movlw	0xff
		movwf	DISP_D4		;zobrazeni na pozici A3
		movlw	0xff
		movwf	DISP_D5		;zobrazeni na pozici A4
		movlw	0xff
		movwf	DISP_D6		;zobrazeni na pozici A5
		movlw	0xff
		movwf	DISP_D7		;zobrazeni na pozici A6
		movlw	0xff
		movwf	DISP_D8		;zobrazeni na pozici A7
		clrf	MIN
		clrf	SEC
		clrf	DOM
		clrf	HOS
		clrf	MENU
		movlw	0x64		;precsaller 100
		movwf	SETINA
		movlw	0x0A		;prescaller 10
		movwf	DESETINA
		movwf	REF_D		;obnovovani ext.displeje
		clrf	POM
		clrf	STAV
		clrf	ZAPAS
		movlw	PULE
		movwf	CISLO
		movwf	POLOCAS
		movlw	B'00000000'
		movwf	TXD0		;externi zobrazeni  SEC 1
		movlw	0x00
		movwf	TXD1		;externi zobrazeni  SEC 10
		movlw	B'10000000'
		movwf	TXD2		;externi zobrazeni  MIN 1
		movlw	0x00
		movwf	TXD3		;externi zobrazeni  MIN 10
		movlw	0x00
		movwf	TXD4		;externi zobrazeni  DOM 10
		movlw	0x00
		movwf	TXD5		;externi zobrazeni  DOM 1
		movlw	0x00
		movwf	TXD6		;externi zobrazeni  HOS 1
		movlw	0x00
		movwf	TXD7		;externi zobrazeni  HOS 10
		bcf	PORTA,0	;###

; **************************************  Obsluha MX displeje	**************************

;###########################################################################

MXDISP		btfss	PIR1,TMR2IF		;multiplex display
		goto	MXDISP			;jump if not
		bcf	PIR1,TMR2IF		;1 kHz
		clrwdt				; reset watchdog

 #if PETR_N > 0
 #if HW==0					;ENKODER s potenciometrem a SMD
		clrw
		btfss	ENKO_P,A_ENKO		;0 1 2 3...
		xorlw	3	
		btfsc	ENKO_P,B_ENKO
		xorlw	1
 
		subwf	SWITCH_1,W		;difference from current value
		andlw	3			;bits 0,1
		bz	ENKO_9			;finish if no change
		andlw	2			;bit 1
		bnz	ENKO_I			;jump if increment
ENKO_D		decf	SWITCH_1		;decrement value
		btfsc	SWITCH_1,0
		bsf	KEYS,TL_L		;sepne TL_L (Decrement)
		goto	ENKO_9			;finish
ENKO_I		incf	SWITCH_1		;increment value
		btfss	SWITCH_1,0
		bsf	KEYS,TL_R		;sepne TL_R (Increment)
ENKO_9
 #else						;ENKODER ALPS
		clrw
		btfss	ENKO_P,A_ENKO		;0 1 2 3...
		xorlw	3
		btfsc	ENKO_P,B_ENKO
		xorlw	1
 
		subwf	SWITCH_1,W		;difference from current value
		andlw	3			;bits 0,1
		bz	ENKO_9			;finish if no change
		andlw	2			;bit 1
		bnz	ENKO_I			;jump if increment
ENKO_D		movf	SWITCH_1,W
		decf	SWITCH_1,F		;decrement value
		andlw	3
		skpnz
		bsf	KEYS,TL_L		;sepne TL_L (Decrement)
		goto	ENKO_9			;finish
ENKO_I		incf	SWITCH_1,F		;increment value
		movf	SWITCH_1,W
		andlw	3
		skpnz
		bsf	KEYS,TL_R		;sepne TL_R (Increment)
ENKO_9
 #endif						;HW!=0
 #else						;original VC
SWA		clrf	SWITCH			;pedpokldej zapnut spnae
		btfsc	ENKO_P,A_ENKO		;je-li opravdu zapnut A, skip
		bsf	SWITCH,0		;indikuj vypnut stav A
		btfsc	ENKO_P,B_ENKO		;je-li opravdu zapnut B, skip
		bsf	SWITCH,1		;indikuj vypnut stav B
		rrf	SWITCH,w		; zarotuje doprava portem SWITCH a ulozi do w
		xorwf	SWITCH,w		; XOR A,B
		movwf	TMP_SW
		btfsc	TMP_SW,0		; je XOR A,B na jednicce?
		bsf	SWITCH,XOR_AB		; ANO - nastav na 1 XOR A,B
		movf	SWITCH_1,w
		xorwf	SWITCH,w
		movwf	TMP_SW	
		btfss	TMP_SW,0		; je zmena na A? (proc ignorujes zmenu na B?)
		goto	SEG
		movf	SWITCH,w
		movwf	SWITCH_1		; zapise historii	
		btfss	SWITCH,XOR_AB		; je na 1 DIR?
		goto	lab3
		bsf	KEYS,TL_R		;sepne TL_R (Increment)
		goto 	SEG
lab3		bsf	KEYS,TL_L		;sepne TL_L (Decrement)
 #endif						;original VC

		DECFSZ	SETINA,F
		goto	SEG
		movlw	0x64			;predelicka 100
		movwf	SETINA
		movfw	MENU
		skpz
		call	CAS			;pri menu <>0		
SEG		clrf	ANODY
		movlw	B'00000111'
		andwf	DIR_DISP,W
SEG_DISP	addwf	PCL,F
		goto	SEG0		;AN0
		goto	SEG1		;AN1
		goto	SEG2		;AN2
		goto	SEG3		;AN3
		goto	SEG4		;AN4
		goto	SEG5		;AN5
		goto	SEG6		;AN6
		goto	SEG7		;AN7
		
SEG0		call	D_MENU	;generovani dat pro displej
		call	TEST_KEYS	;test klaves
		bcf	KEYS,TL_L
		bcf	KEYS,TL_R
		movf	DISP_D1,w	
		movwf	KATODY
		bsf	ANODY,AN0
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD4		;1.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG1		movf	DISP_D2,w			
		movwf	KATODY
		bsf	ANODY,AN1
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD5		;2.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG2		movf	DISP_D3,w			
		movwf	KATODY
		bsf	ANODY,AN2
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD6		;3.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG3		movf	DISP_D4,w		
		movwf	KATODY
		bsf	ANODY,AN3
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD7		;4.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG4		movf	DISP_D5,w		
		movwf	KATODY
		bsf	ANODY,AN4
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD0		;5.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG5		movf	DISP_D6,w		
		movwf	KATODY
		bsf	ANODY,AN5
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD1		;6.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG6		movf	DISP_D7,w		
		movwf	KATODY
		bsf	ANODY,AN6
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	DEND
		movfw	TXD2		;7.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
SEG7		movf	DISP_D8,w		
		movwf	KATODY
		bsf	ANODY,AN7
		clrf	DIR_DISP
		movf	REF_D,W		;je REF_D nula?
		btfss	STATUS,Z
		goto	Test_TIM1
		movfw	TXD4		;8.zobrazovane na ext. displeji
		movwf	SSPBUF		
		goto	DEND
		goto	Test_TIM1
DEND		incf	DIR_DISP,f
			
Test_TIM1	btfss	TIM,0		; test casove smycky T0
		goto	Test_TIM2
		decfsz	TIMR1,F
		goto	Test_TIM2
		bcf	TIM,0
Test_TIM2	btfss	TIM,1		; test casove smycky T1
		goto	Test_TIM3
		decfsz	TIMR2L,F
		goto	Test_TIM3
		movlw	0x20		; casova smycka (100ms) 
		movwf	TIMR2L		
		decfsz	TIMR2H,F
		goto	Test_TIM3
		bcf	TIM,1
Test_TIM3	btfss	TIM,2		; test casove smycky T2
		goto	MXDISP
		decfsz	TIMR3L,F
		goto	MXDISP
		decfsz	TIMR3M,F
		goto	MXDISP
		movlw	TIME_OUTL
		movwf	TIMR3M
		decfsz	TIMR3H,F
		goto	MXDISP
		bcf	TIM,2
		clrf	MENU
		movfw	POKUS
		bnz	MXDISP	
		incf	POKUS,F
		goto	MXDISP		;formalni konec programu

;--------------------	MENU 00	-------------------------------------------------------------------------	
MENU_0		movlw	0x0F			;
		call	BCD_DISP
		movwf	DISP_D1
		movlw	0x11
		call	BCD_DISP
		movwf	DISP_D2
		movlw	0x15
		call	BCD_DISP
		movwf	DISP_D3
		movlw	0x15
		call	BCD_DISP
		movwf	DISP_D4
		movlw	0x02
		call	BCD_DISP
		movwf	DISP_D5
		movlw	0x13
		call	BCD_DISP
		movwf	DISP_D6
		movfw	CISLO
		call	HEX_BCD_1b
		movfw	D10
		movwf	DISP_D7
		movfw	D1
		movwf	DISP_D8
		btfsc	KEYS,TL_R		;neni stisknuto Tl R	
		goto	Mn0_plus
		btfsc	KEYS,TL_L		;neni stisknuto Tl L	
		goto	Mn0_minus
		btfsc	KEYS,TL_S		;stisknuto Tl enkoderu
		goto	Mn0_sw	
		return
Mn0_plus	movlw	PULE		;45 min
		subwf	CISLO,w
		BZ	Pol_15
		movlw	0x05
		addwf	CISLO,f
		return
Pol_15		movlw	0x0F
		movwf	CISLO
		return
Mn0_minus	movlw	0x0F		;15 min
		subwf	CISLO,w
		BZ	Pol_45
		movlw	0x05
		subwf	CISLO,f
		return
Pol_45		movlw	PULE
		movwf	CISLO
		return		
Mn0_sw		movfw	CISLO
		movwf	POLOCAS
		movfw	MENU
		movwf	MENU_TMP
		movlw	0x09
		movwf	MENU
		call	TIMER1
		return

;--------------------	MENU 09 - F O T B A L 	---------------------------------------------------------------------	
MENU_09	bcf	PORTA,0	;###
		btfsc	ZAPAS,6
		bsf	PORTA,0	;###

		btfsc	KEYS_LONG,TL_MIN	;nastaveni 2.polocasu
		goto	Mn09_LOM
		btfsc	KEYS,TL_MIN
		goto	Mn09_MIN
		btfsc	KEYS,TL_SEC
		goto	Mn09_SEC
		btfsc	KEYS,TL_DOM
		goto	Mn09_DOM
		btfsc	KEYS,TL_HOS
		goto	Mn09_HOS
		btfsc	KEYS_LONG,TL_S		
		goto	Mn09_LOSW
		btfsc	KEYS,TL_S		
		goto	Mn09_SW
		btfsc	KEYS,TL_R		
		goto	Mn09_R
		btfsc	KEYS,TL_L		
		goto	Mn09_L
Mn09_TST	btfss	ZAPAS,5			;je 1.polocas?
		goto	Mn09_TST1
		movfw	POLOCAS
		subwf	MIN,W
		skpnz		
		bsf	ZAPAS,7			;zapne blikani
Mn09_TST1	btfss	ZAPAS,6			;je 2.polocas?
		return
		rrf	POLOCAS,W
		subwf	MIN,W
		skpnz		
		bsf	ZAPAS,7			;zapne blikani
		return
	
Mn09_SW	btfsc	KEYS,TL_MIN		;stisknuto TL_MIN
		incf	MIN,F
		btfsc	KEYS,TL_SEC
		clrf	SEC
		btfsc	KEYS,TL_DOM
		clrf	DOM
		btfsc	KEYS,TL_HOS
		clrf	HOS
		return

Mn09_LOSW	BTFSS	POM,6		;testuje START-STOP
		goto	Mn09_St
		BCF	POM,6		;STOP
		BCF	ZAPAS,7		;vypne BLIK_M
		call	TIMER1
		return
Mn09_St	BSF	POM,6		;START
		bsf	ZAPAS,5		;nastavi priznak 1.polocasu
		call	TIMER1
		return


Mn09_R		return
Mn09_L		return

Mn09_LOM	btfsc	POM,6		;stisknuto LONG_MIN - jen pri stopnutem case
		return
		movfw	POLOCAS
		movwf	MIN
		clrf	SEC
		bcf	ZAPAS,5		;nuluje priznak 1.polocasu
		bsf	ZAPAS,6		;nastavi priznak 2.polocasu
		bcf	ZAPAS,7		;vypne blikani
		return

Mn09_MIN	btfsc	KEYS,TL_R	; stisknuto TL MIN
		incf	MIN,F
		btfsc	KEYS,TL_L	
		decf	MIN,F
		btfsc	KEYS,TL_LONG	
		clrf	MIN
		return

Mn09_SEC	btfsc	KEYS,TL_R	; stisknuto TL SEC
		incf	SEC,F
		btfsc	KEYS,TL_L	
		decf	SEC,F
		btfsc	KEYS,TL_LONG	
		clrf	SEC
		return	

Mn09_DOM	btfsc	KEYS,TL_R	;stisknuto TL DOM
		incf	DOM,F
		btfsc	KEYS,TL_L	
		decf	DOM,F
		btfsc	KEYS,TL_LONG	
		clrf	DOM
		return	

Mn09_HOS	btfsc	KEYS,TL_R	; stisknuto TL HOS
		incf	HOS,F
		btfsc	KEYS,TL_L	
		decf	HOS,F
		btfsc	KEYS,TL_LONG	
		clrf	HOS
		return	

; ----------------- E N D   ME N U    ---------------------------------------------------------------------------------------

CAS		BTFSS	POM,6		;testuje START
		goto	SEC_0
		DECFSZ	DESETINA,F
		goto	D_CAS
		movlw	0x0a
		movwf	DESETINA
		BCF	TIM,7		;vypnout blikani           
		incf	SEC,F	
		movfw	SEC
		sublw	0x3C		;testuje 60
		BTFSS	STATUS,Z
		GOTO	D_CAS
		incf	MIN,F		;pricte Minutu
		clrf	SEC
		call	TIMER3		;refresh
		BTFSS	ZAPAS,5		;je 1.polocas?
		goto	POL_2
		movf	MIN,W
		SUBLW	POLOCAS	;test na 45.minutu  0x2D
		BTFSS	STATUS,Z
		goto	POL_2
		BSF	ZAPAS,7		;ma blikat MINUTA
		bsf	PORTA,0	;###
		goto	D_CAS
POL_2		BTFSS	ZAPAS,6		;je 2.polocas?
		goto	D_CAS
		movf	MIN,W
		SUBLW	POLOCAS*2	;test na 90.minutu  0x5A
		BTFSS	STATUS,Z
		goto	D_CAS
		BSF	ZAPAS,7		;ma blikat MINUTA

D_CAS		movf	DESETINA,W
		SUBLW	0x05
		BTFSS	STATUS,Z
		goto	SEC_0
		BSF	TIM,7
SEC_0		movf	SEC,W		;Zobrazeni na displeji
		CALL	EXT_BCD
		movf	D10,W
		movwf 	TXD1		;EXT SEC10
		movf	D1,W
		movwf	TXD0		;EXT SEC1
;	------------------------------------------
		movf	SEC,W
		CALL	HEX_BCD_1b
		movf	D10,W
		movwf 	DISP_D3
		movf	D1,W
		movwf	DISP_D4

MIN_0		movf	MIN,W		;EXTERNI DISPLAY
		CALL	EXT_BCD
		movf	D10,W
		movwf 	TXD3		;MIN10
		movf	D1,W
		movwf	TXD2		;MIN1
		BTFSS	ZAPAS,7
		goto	IMIN_0
		BTFSS	TIM,7
		goto	IMIN_0
		movlw	0x00
		movwf	TXD3		;MIN10
		movwf	TXD2		;MIN1		;#####
;-------------------------------------------------
IMIN_0		movf	MIN,W		;INTERNI
		CALL	HEX_BCD_1b
		movf	D10,W
		movwf 	DISP_D1		;10min
		movf	D1,W
		movwf	DISP_D2		;1min
		BTFSS	ZAPAS,7
		goto	T_BLIK
		BTFSS	TIM,7
		goto	T_BLIK
		movlw	0xFF
		movwf	DISP_D1
		movwf	DISP_D2		;#####

T_BLIK		BTFSS	TIM,7		;testuje blikani 
		goto	DOM_0
		movf	DISP_D2,w
		ANDLW	B'11111011'	;rozsviti tt
		movwf	DISP_D2
		movf	TXD3,W		;MIN10,w
		ADDLW	SPE		;rozsviti tte
		movwf	TXD3		;MIN10
		movf	TXD2,W		;MIN1,w
		ADDLW	SPE		;rozsviti tte
		movwf	TXD2		;MIN1

DOM_0		movf	DOM,W		;EXTERNI DISPLAY
		CALL	EXT_BCD
		movf	DOM,W
		sublw	0x09
		BTFSS	STATUS,C
		goto	EDOM_10
		movlw	0x00
		movwf	TXD4		;DOM10
		goto	EDOM_09
EDOM_10	movf	D10,W		;potlaceni nevyznamne nuly
		movwf 	TXD4		;DOM10
EDOM_09	movf	D1,W
		movwf	TXD5		;DOM1
		BTFSS	POM,6		;testuje START STOP
		goto	EDOM_0
		BTFSC	TIM,7		;testuje blikani 
		goto	EDOM_0
		movf	TXD5,W		;DOM1,w
		ADDLW	SPE		;rozsviti tte
		movwf	TXD5		;DOM1
		movf	TXD4,W		;DOM10,w
		ADDLW	SPE		;rozsviti tte
		movwf	TXD4		;DOM10
;-------------------------------------------
EDOM_0		movf	DOM,W
		CALL	HEX_BCD_1b
		movf	DOM,W
		sublw	0x09
		BTFSS	STATUS,C
		goto	DOM_10
		movlw	0xff
		movwf	DISP_D5
		goto	DOM_09
DOM_10		movf	D10,W		;potlaceni nevyznamne nuly
		movwf 	DISP_D5
DOM_09		movf	D1,W
		movwf	DISP_D6
		BTFSS	POM,6		;testuje START STOP
		goto	EHOS_0
		movf	DISP_D6,w
		ANDLW	B'11111011'	;rozsviti tt
		movwf	DISP_D6

EHOS_0		movf	HOS,W
		CALL	EXT_BCD
		movf	HOS,W
		sublw	0x09
		BTFSS	STATUS,C
		goto	EHOS_10
		movlw	0x00
		movwf	TXD7		;HOS1
		movf	D1,W
		movwf	TXD6		;HOS10
		goto	HOS_0
EHOS_10	movfw	D10		;potlaceni nevyznamne nuly + posuv
		movwf 	TXD6		;HOS10
		movf	D1,W
		movwf	TXD7		;HOS1
;	--------------------------
HOS_0		movf	HOS,W
		CALL	HEX_BCD_1b
		movf	HOS,W
		sublw	0x09
		BTFSS	STATUS,C
		goto	HOS_10
		movlw	0xff
		movwf	DISP_D8
		movf	D1,W
		movwf	DISP_D7
		goto	ECAS
HOS_10		movfw	D10		;potlaceni nevyznamne nuly + posuv
		movwf 	DISP_D7
		movf	D1,W
		movwf	DISP_D8
ECAS		RETURN

; ..............................  T I M E R S  ...........................................
TIMER1		movlw	0xFF		; casova smycka nn x TMR2IF (1.00ms)
		movwf	TIMR1		;0.25s
		bsf	TIM,0
		return
TIMER2		movlw	0x64		; casova smycka (100ms) 
		movwf	TIMR2L		
		movlw	TIME_DISP		
		movwf	TIMR2H		;100ms x TMR2IF  (1sec)
		bsf	TIM,1
		return
TIMER3		movlw	0xff		; casova smycka (cca 0.5s)
		movwf	TIMR3L
		movlw	TIME_OUTL	;0x0A => 5sec
		movwf	TIMR3M
		movlw	TIME_OUT	;5* TIME_OUT (sec)
		movwf	TIMR3H
		bsf	TIM,2
		return	

;--------------------	T L A C I T K A 	------------------------------------------------------------------------
TEST_KEYS	movlw	B'00000011'		;maska pznak tlatek
		andwf	KEYS,F			;vynuluj pznaky tlatek

SWS		bsf	KEYS,TL_S		;nastav priznal tlacitka TL_S
		btfss	ENKO_P,S_ENKO		;je zapnut S?
		btfsc	TIM,0			;a zrove pznak asovae?
		goto	Mns		
		decfsz	T_LONG,F		;testuje dlouhy stisk 
		goto	Test_MIN
		bsf	KEYS_LONG,TL_S		;zapne priznak dlouheho stisku Tl 
		goto	Test_MIN
Mns		movlw	TIM_LONG
		movwf	T_LONG
		bcf	KEYS,TL_S		;ne, vynuluj pznak tlatka TL_S
		bcf	KEYS_LONG,TL_S		;vypne priznak dlouheho stisku Tl 

Test_MIN	bsf	KEYS,TL_MIN		;nastav priznal tlacitka TL_S
		btfss	SW_PORT,SW_MIN	;je zapnut TL_MIN?
		btfsc	TIM,0			;a zrove pznak asovae?
		goto	Mnm		
		decfsz	T_LONGM,F		;testuje dlouhy stisk 
		goto	Test_SEC
		bsf	KEYS_LONG,TL_MIN	;zapne priznak dlouheho stisku Tl 
		goto	Test_SEC
Mnm		movlw	TIM_LONG
		movwf	T_LONGM
		bcf	KEYS,TL_MIN		;ne, vynuluj pznak tlatka TL_S
		bcf	KEYS_LONG,TL_MIN	;vypne priznak dlouheho stisku Tl 

Test_SEC	btfsc	SW_PORT,SW_SEC		;Tl SEC
		goto	Test_DOM
		bsf	KEYS,TL_SEC
Test_DOM	btfsc	SW_PORT,SW_DOM		;Tl DOM
		goto	Test_HOS
		bsf	KEYS,TL_DOM
Test_HOS	btfsc	SW_PORT,SW_HOS		;Tl HOS
		goto	Test_REFRESH
		bsf	KEYS,TL_HOS

Test_REFRESH	movlw	B'10010111'
		andwf	KEYS,w
		bnz	TMR_REF
		return		
; =========  konec TL_TEST  ============

TMR_REF	call	TIMER3
		return

HEX_BCD_1b	movwf	TMP_L		; W -> TMP_L ,vrati v BCD formatu D100, D10 a D1	
		clrf	CISLA32
		clrf	CISLA10
RAD100		movlw	.100
		subwf	TMP_L,W
		btfss	STATUS,C
		goto	RAD10		; C=0, vysledek je zaporny konec odcitani
		movwf	TMP_L
		incf	CISLA32,F
		goto	RAD100
RAD10		movlw	.10
		subwf	TMP_L,W
		btfss	STATUS,C
		goto	RAD1		; C=0, vysledek je zaporny konec odcitani
		movwf	TMP_L
		incf	CISLA10,F
		goto	RAD10
RAD1		swapf	CISLA10,W	;secte CISLA10 + TMP_L a ulozi do W a CISLA10
		iorwf	TMP_L,W
		movwf	CISLA10
		movf	CISLA32,w	;prevod CISLA32 na DEC100
		andlw	0x0f		; maskuje horni 4bity
		call	BCD_DISP
		movwf	D100
		movf	CISLA10,w	;prevod CISLA10(L) na DEC1
		andlw	0x0f		; maskuje horni 4bity
		call	BCD_DISP
		movwf	D1
		swapf	CISLA10,f	; prohod BCD u CISLA10
		movf	CISLA10,w	; preved CISLA10 na DEC10
		andlw	0x0f		; maskuje horni 4bity
		call	BCD_DISP
		movwf	D10
		return

EXT_BCD	nop			;vrati v BCD formatu D100, D10 a D1
		movwf	TMP_L		; W -> TMP_L	
		clrf	CISLA32
		clrf	CISLA10
ERAD100	movlw	.100
		subwf	TMP_L,W
		btfss	STATUS,C
		goto	ERAD10		; C=0, vysledek je zaporny konec odcitani
		movwf	TMP_L
		incf	CISLA32,F
		goto	ERAD100
ERAD10		movlw	.10
		subwf	TMP_L,W
		btfss	STATUS,C
		goto	ERAD1		; C=0, vysledek je zaporny konec odcitani
		movwf	TMP_L
		incf	CISLA10,F
		goto	ERAD10
ERAD1		swapf	CISLA10,W	;secte CISLA10 + TMP_L a ulozi do W a CISLA10
		iorwf	TMP_L,W
		movwf	CISLA10
		movf	CISLA32,w	;prevod CISLA32 na DEC100
		andlw	0x0f		; maskuje horni 4bity
		call	SEG7_EXT
		movwf	D100
		movf	CISLA10,w	;prevod CISLA10(L) na DEC1
		andlw	0x0f		; maskuje horni 4bity
		call	SEG7_EXT
		movwf	D1
		swapf	CISLA10,f	; prohod BCD u CISLA10
		movf	CISLA10,w	; preved CISLA10 na DEC10
		andlw	0x0f		; maskuje horni 4bity
		call	SEG7_EXT
		movwf	D10
		return

HEX_BCD_2b	swapf	CSC2,W		;pevede CSC1 a CSC2 na BCD 1000, 100, 10 a 1
		iorlw	0xF0
		movwf	TISIC
		addwf	TISIC,F
		addlw	0xE2
		movwf	STO
		addlw	0x32
		movwf	JEDNA

		movf	CSC2,W
		andlw	0x0F
		addwf	STO,F
		addwf	STO,F
		addwf	JEDNA,F
		addlw	0xE9
		movwf	DESET
		addwf	DESET,F
		addwf	DESET,F

		swapf	CSC1,W
		andlw	0x0F
		addwf	DESET,F
		addwf	JEDNA,F

		rlf	DESET,F
		rlf	JEDNA,F
		comf	JEDNA,F
		rlf	JEDNA,F

		movf	CSC1,W
		andlw	0x0F
		addwf	JEDNA,F
		rlf	TISIC,F

		movlw	0x07
		movwf	DTISIC
		movlw	0x0A
;       ------------------------
LB1		addwf	JEDNA,F
		decf	DESET,F
		btfss	STATUS,C
		goto	LB1
;       ------------------------
LB2		addwf	DESET,F
		decf	STO,F
		btfss	STATUS,C
		goto	LB2
;       ------------------------
LB3		addwf	STO,F
		decf	TISIC,F
		btfss	STATUS,C
		goto	LB3
;       ------------------------
LB4		addwf	TISIC,F
		decf	DTISIC,F
		btfss	STATUS,C
		goto	LB4
;       ------------------------
		movf	JEDNA,W
		andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
		call	BCD_DISP
		movwf	DISP_D8
		movf	DESET,W
		andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
		call	BCD_DISP
		movwf	DISP_D7
		movf	STO,W
		andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
		call	BCD_DISP
		movwf	DISP_D6
		movf	TISIC,W
		andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
		call	BCD_DISP
		movwf	DISP_D5
		return	
	
;=======================================================	
		END			; Konec programu

