
;****************************************************************************
;		File Name:		FOTBALEK
;		Date:			3.3.2014
;		File Version:	
;		HW: 			board "FOTBAL_II"
;		Author:			VC - Allcomp
;
;****************************************************************************
;		Description: 	Ridici jednotka pro tabuli fotbal
;					
;		 Port : A0 - Temperature	B0 - A0 (10MIN)
;               A1 - Svetlo			B1 - A1 (1MIN)
;               A2 - TL-MIN			B2 - A2 (10SEC)
;               A3 - TL-SEC			B3 - A3 (1SEC)
;               A4 - TL-DOM			B4 - A4 (10DOM)
;               A5 - TL-HOS			B5 - A5 (1DOM)
;               A6 - X2 4000 kHz	B6 - A6 (10HOS)
;               A7 - X2				B7 - A7 (1HOS)
;
;		 Port : C0 - (Terc A) +DO	D0 - seg D
;               C1 - (Terc B) +DO	D1 - seg C
;               C2 - PWM			D2 - seg P
;               C3 - SCL            D3 - seg E
;               C4 - SDI (pistole)	D4 - seg B
;               C5 - SDO			D5 - seg A
;               C6 - Tx				D6 - seg F
;               C7 - Rx				D7 - seg G
;
;               E0 - enkoder SW		E2 - enkoder A
;               E1 - enkoder B		E3 - MCLR
;===========================================================================
;	V3.00	- 	TMR2IF= 1ms
;   V3.02   - vyukovy program
;
;*************** Definice promennych ***************************************
#define LED_G   PORTC,0         ;LED
#define SW_MIN  PORTA,2         ;TL_MIN
#define SW_SEC  PORTA,3         ;TL_SEC
#define SW_DOM  PORTA,4         ;TL_DOM
#define SW_HOS  PORTA,5         ;TL_HOS
; ********** Hlavicka programu **********************************************
	list		p=16f1937           ; Typ mikrokontroleru
	#include	<p16f1937.inc>	; Definice SFR registru a bit		
	errorlevel	-302		; Zakazat hlaseni"Register..bank 0.."
;
; *********** Program Configuration Registers *******************************
	__CONFIG    _CONFIG1,_FOSC_INTOSC
    __CONFIG	_CONFIG2,_BORV_HI
;
		org	0x000		; Addresa prvni instrukce
		goto 	START
		org	0x004		; vektor preruseni
;
; ************************   HLAVNI PROGRAM   ********************************
START           movlb   0x3             ;>  BANK 3 <
                movlw   B'00000000'     ;1-analog  0-digital
                movwf   ANSELA
                clrf	ANSELB
                clrf	ANSELD
                clrf	ANSELE
                movlb   0x1             ;>  BANK 1
                movlw	0xF9            ;0xFA (250)
                movwf	PR2             ;preddelicka na 250
                movlw	B'10111000'
                movwf	OPTION_REG      ; TIMER na PA4
                movlw	B'11111111'     ;
                movwf	TRISA           ; 0-OUT; 1-IN
                movlw	B'00000000'     ;
                movwf	TRISB           ; 0-OUT; 1-IN
                movlw	B'10000000'
                movwf	TRISC           ; 0-OUT; 1-IN
                clrf	TRISD           ; 0-OUT; 1-IN
                movlw	B'1111'
                movwf	TRISE           ; 0-OUT; 1-IN
                movlw   B'01111010'
                movwf   OSCCON          ;16MHz
                movlb   0x0             ;>  BANK 0 <
                movlw   B'01110100'     ; Presc 1:5 Postsc 1:16  PR2 1:136 (1/704)
                movwf   T2CON           ; Nastaveni TMR2
                movlw	0xff
                movwf	DISP_D1         ;zobrazeni na pozici A0
                movlw	0xff
                movwf	DISP_D2         ;zobrazeni na pozici A1
                movlw	0xff
                movwf	DISP_D3         ;zobrazeni na pozici A2
                movlw	0xff
                movwf	DISP_D4         ;zobrazeni na pozici A3
                movlw	0xff
                movwf	DISP_D5         ;zobrazeni na pozici A4
                movlw	0xff
                movwf	DISP_D6         ;zobrazeni na pozici A5
                movlw	0xff
                movwf	DISP_D7         ;zobrazeni na pozici A6
                movlw	0xff
                movwf	DISP_D8         ;zobrazeni na pozici A7
                clrf	MIN
                clrf	SEC
                clrf	MENU
                movlw	0x64            ;precsaller 100
                movwf	SETINA
                movlw	0x0A            ;prescaller 10
                movwf	DESETINA
                movwf	CISLO
;
; ***************  Hlavni smycka - obsluha MX displeje	**************************
MXDISP          btfss	PIR1,TMR2IF		;multiplex display
                goto	MXDISP			;jump if not
                bcf     PIR1,TMR2IF		;1 kHz
SWA             clrf	SWITCH			;pedpokldej zapnut spnae
                btfsc	ENKO_P,A_ENKO	;je-li opravdu zapnut A, skip
                bsf     SWITCH,0		;indikuj vypnut stav A
                btfsc	ENKO_P,B_ENKO	;je-li opravdu zapnut B, skip
                bsf     SWITCH,1		;indikuj vypnut stav B
                rrf     SWITCH,w		; zarotuje doprava portem SWITCH a ulozi do w
                xorwf	SWITCH,w		; XOR A,B
                movwf	TMP_SW
                btfsc	TMP_SW,0		; je XOR A,B na jednicce?
                bsf     SWITCH,XOR_AB	; ANO - nastav na 1 XOR A,B
                movf	SWITCH_1,w
                xorwf	SWITCH,w
                movwf	TMP_SW
                btfss	TMP_SW,0		; je zmena na A? (proc ignorujes zmenu na B?)
                goto	CLOCK1
                movf	SWITCH,w
                movwf	SWITCH_1		; zapise historii
                btfss	SWITCH,XOR_AB	; je na 1 DIR?
                goto	lab3
                bsf     KEYS,TL_R		;sepne TL_R (Increment)
                goto 	CLOCK1
lab3            bsf     KEYS,TL_L		;sepne TL_L (Decrement)
;----------------------------------------------------------------
CLOCK1          bcf     CLOCK,0         ;reset desetiny
                bcf     CLOCK,1         ;reset vteriny
                decfsz  SETINA
                goto    ADD_CSC
                bsf     CLOCK,0         ;platna desetina
                movlw   .100
                movwf   SETINA
                decfsz  DESETINA
                goto    ADD_CSC
                bsf     CLOCK,1          ;platna vterina
                movlw   .10             ;1 sec impulz na RA7
                movwf   DESETINA
ADD_CSC         btfss   CLOCK,0
                goto    Tlac_SW
                btfsc   TIM,7
                goto    Tlac_SW
                incfsz	CSA1,F
                goto    Tlac_SW
                incf	CSA2,F
Tlac_SW         btfss   KEYS,TL_S
                goto    Tlac_MIN
                bcf     TIM,7
                clrf    CSA1
                clrf    CSA2
Tlac_MIN        btfss   KEYS,TL_MIN
                goto    Tlac_SEC
                bsf     TIM,7
Tlac_SEC        btfss   KEYS,TL_SEC
                goto    Tlac_DOM
                bsf     TIM,7
Tlac_DOM        btfss   KEYS,TL_DOM
                goto    Tlac_HOS
                bsf     TIM,7
Tlac_HOS        btfss   KEYS,TL_HOS
                goto    SEG
                bsf     TIM,7
SEG             bcf     LED_G           ;
                btfsc   TIM,7           ;
                bsf     LED_G           ;
                clrf	ANODY
                movlw	B'00000111'
                andwf	DIR_DISP,W
SEG_DISP        brw
                goto	SEG0		;AN0
                goto	SEG1		;AN1
                goto	SEG2		;AN2
                goto	SEG3		;AN3
                goto	SEG4		;AN4
                goto	SEG5		;AN5
                goto	SEG6		;AN6
                goto	SEG7		;AN7
;-------------------------------------------------------------------
SEG0            call	D_MENU          ;generovani dat pro displej
                movf	DISP_D1,w
                movwf	KATODY
                bsf     ANODY,AN0
                goto	DEND
SEG1            movf	DISP_D2,w
                movwf	KATODY
                call	TEST_KEYS       ;test klaves
                bcf     KEYS,TL_L
                bcf     KEYS,TL_R
                bsf     ANODY,AN1
                goto	DEND
SEG2            movf	DISP_D3,w
                movwf	KATODY
                bsf     ANODY,AN2
                goto	DEND
SEG3            movf	DISP_D4,w
                movwf	KATODY
                bsf     ANODY,AN3
                goto	DEND
SEG4            movf	DISP_D5,w
                movwf	KATODY
                bsf     ANODY,AN4
                goto	DEND
SEG5            movf	DISP_D6,w
                movwf	KATODY
                bsf     ANODY,AN5
                goto	DEND
SEG6            movf	DISP_D7,w
                movwf	KATODY
                bsf     ANODY,AN6
                goto	DEND
SEG7            movf	DISP_D8,w
                movwf	KATODY
                bsf     ANODY,AN7
                clrf	DIR_DISP
                goto    Test_TIM1
DEND            incf	DIR_DISP,f
                incf    CSC1,F
;-------------------------------------------------------------------
Test_TIM1       btfss	TIM,0		; test casove smycky T0
                goto	Test_TIM2
                decfsz	TIMR1,F
                goto	Test_TIM2
                bcf     TIM,0
Test_TIM2       btfss	TIM,1		; test casove smycky T1
                goto	FKP
                decfsz	TIMR2L,F
                goto	FKP
                movlw	0x20		; casova smycka (100ms)
                movwf	TIMR2L
                decfsz	TIMR2H,F
                goto	FKP
                bcf     TIM,1
                bcf     TIM,2
FKP             goto	MXDISP		;formalni konec programu
;***********************************************************************************
;***********************************************************************************
;
;	===================  P O D P R O G R A M Y    cast 1   =========================
D_MENU          movfw	MENU
                andlw	B'00001111'	;omezeni skoku max 0x1F
                brw
                goto	MENU_0          ;0x00
                goto	MENU_0          ;0x01
                goto	MENU_0          ;0x02
                goto	MENU_0          ;0x03
                goto	MENU_0          ;0x04
                goto	MENU_0          ;0x05
                goto	MENU_0          ;0x06
                goto	MENU_0          ;0x07
                goto	MENU_0          ;0x08
                goto	MENU_0          ;0x09 FOTBAL
;-------------------------------------------------------------------
BCD_DISP        andlw	B'00111111'             ;omezeni skoku max 0x1F
                brw                             ;prevod BCD kod 
                retlw	SG+SP                   ;0
                retlw	SA+SD+SE+SF+SG+SP       ;1
                retlw	SC+SF+SP                ;2
                retlw	SE+SF+SP                ;3
                retlw	SA+SD+SE+SP             ;4
                retlw	SB+SE+SP                ;5
                retlw	SB+SP                   ;6
                retlw	SD+SE+SF+SG+SP          ;7
                retlw	SP                      ;8
                retlw	SE+SP                   ;9
                retlw	SD+SP                   ;A
                retlw	SA+SB+SP                ;b
                retlw	SB+SC+SG+SP             ;C
                retlw	SA+SF+SP                ;d
                retlw	SB+SC+SP                ;E
                retlw	SB+SC+SD+SP             ;F
                retlw	SA+SB+SC+SF+SP          ;c	0x10
                retlw	SA+SB+SF+SP             ;o	0x11
                retlw	SA+SB+SC+SD+SF+SP		;r	0x12
                retlw	SA+SB+SC+SD+SE+SF+SP	;-	0x13
                retlw	SA+SE+SP                ;Y	0x14
                retlw	SA+SB+SC+SP             ;t	0x15
                retlw	SA+SB+SC+SG+SP          ;L	0x16
                retlw	SA+SB+SF+SG+SP          ;u	0x17
                retlw	SC+SD+SP                ;P	0x18
                retlw	SA+SB+SD+SF+SP          ;n	0x19
                retlw	SA+SB+SD+SG+SP          ;li	0x1A
                retlw	SA+SD+SP                ;H	0x1B
                retlw	SA+SD+SG+SP             ;II	0x1C
                retlw	SA+SB+SC+SD+SG+SP		;l	0x1D
                retlw	SA+SF+SG+SP             ;J	0x1E
                retlw	0xFF                    ;	0x1F
;-----------	MENU 00	---------------------------------------------------------------
MENU_0          movlw	0x0A			;A
                call	BCD_DISP
                movwf	DISP_D1
                bcf     DISP_D1,SP
                movlw	0x1B            ;H
                call	BCD_DISP
                movwf	DISP_D2
                movlw	0x00            ;0
                call	BCD_DISP
                movwf	DISP_D3
                movlw	0x1E            ;J
                call	BCD_DISP
                movwf	DISP_D4
                call	HEX_BCD_2b
                return
; ----------------- E N D   ME N U    --------------------------------------------------
; ..............................  T I M E R S  .........................................
TIMER1          movlw	0xFF		; casova smycka nn x TMR2IF (1.00ms)
                movwf	TIMR1		;0.25s
                bsf     TIM,0
                return
TIMER2          movlw	0x64		; casova smycka (100ms)
                movwf	TIMR2L
                movlw	TIME_DISP
                movwf	TIMR2H		;100ms x TMR2IF  (1sec)
                bsf     TIM,1
                return
;--------------------	T L A C I T K A 	------------------------------------------------
TEST_KEYS       btfsc	TIM,0
                return
                movlw	B'00000011'             ;maska pznak tlatek
                andwf	KEYS,F                  ;vynuluj pznaky tlatek
SWS             btfsc	ENKO_P,S_ENKO           ;je zapnut S?
                goto    Test_MIN
                bsf     KEYS,TL_S               ;nastav pznak tlatka TL_S
                goto    TIMER1
Test_MIN        btfsc	SW_MIN                  ;je zapnut TL_MIN?
                goto    Test_SEC
                bsf     KEYS,TL_MIN             ;nastav pznak tlatka TL_S
                goto    TIMER1
Test_SEC        btfsc	SW_SEC                  ;Tl SEC
                goto	Test_DOM
                bsf     KEYS,TL_SEC
                goto    TIMER1
Test_DOM        btfsc	SW_DOM                  ;Tl DOM
                goto	Test_HOS
                bsf     KEYS,TL_DOM
                goto    TIMER1
Test_HOS        btfsc	SW_HOS                  ;Tl HOS
                return
                bsf     KEYS,TL_HOS
                goto    TIMER1
; ----------- Prepocet HEX na BCD  ----------------------------------------------
HEX_BCD_2b      swapf	CSA2,W		;pevede CSC1 a CSC2 na BCD 1000, 100, 10 a 1
                iorlw	0xF0
                movwf	TISIC
                addwf	TISIC,F
                addlw	0xE2
                movwf	STO
                addlw	0x32
                movwf	JEDNA
                movf	CSA2,W
                andlw	0x0F
                addwf	STO,F
                addwf	STO,F
                addwf	JEDNA,F
                addlw	0xE9
                movwf	DESET
                addwf	DESET,F
                addwf	DESET,F
                swapf	CSA1,W
                andlw	0x0F
                addwf	DESET,F
                addwf	JEDNA,F
                rlf     DESET,F
                rlf     JEDNA,F
                comf	JEDNA,F
                rlf     JEDNA,F
                movf	CSA1,W
                andlw	0x0F
                addwf	JEDNA,F
                rlf     TISIC,F
                movlw	0x07
                movwf	DTISIC
                movlw	0x0A
;       ------------------------
LB1             addwf	JEDNA,F
                decf	DESET,F
                btfss	STATUS,C
                goto	LB1
;       ------------------------
LB2             addwf	DESET,F
                decf	STO,F
                btfss	STATUS,C
                goto	LB2
;       ------------------------
LB3             addwf	STO,F
                decf	TISIC,F
                btfss	STATUS,C
                goto	LB3
;       ------------------------
LB4             addwf	TISIC,F
                decf	DTISIC,F
                btfss	STATUS,C
                goto	LB4
;       ------------------------
                movf	JEDNA,W
                andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
                call	BCD_DISP
                movwf	DISP_D8
                movf	DESET,W
                andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
                call	BCD_DISP
                movwf	DISP_D7
                movf	STO,W
                andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
                call	BCD_DISP
                movwf	DISP_D6
                movf	TISIC,W
                andlw	0x0f		;operace AND mezi W a konst. => W '0000xxxx'
                call	BCD_DISP
                movwf	DISP_D5
                return
;===================== Konec podprogramu ==========================================
;
;	*************    DEFINING PROGRAM VARIABLES   *************************
ENKO_P          EQU	PORTE		;
A_ENKO          EQU	0x2
B_ENKO          EQU	0x1
S_ENKO          EQU	0x0
;
ANODY           EQU	PORTB		;nastaveni adresy anod displeje
AN0             EQU	0x0
AN1             EQU	0x1         ;nastaveni jednotlivych anod
AN2             EQU	0x2
AN3             EQU	0x3
AN4             EQU	0x4
AN5             EQU	0x5
AN6             EQU	0x6
AN7             EQU	0x7
;
KATODY          EQU	PORTD		;0x008 nastaveni katod displeje
SA              EQU	0x20		;nastaveni segmentu displeje
SB              EQU	0x10
SC              EQU	0x02
SD              EQU	0x01
SE              EQU	0x08
SF              EQU	0x40
SG              EQU	0x80
SP              EQU	0x04
;	REGISTER KEYS:
TL_R            EQU	0x0
TL_L            EQU	0x1
TL_S            EQU	0x2
TL_LONG         EQU	0x3
TL_MIN          EQU	0x4
TL_SEC          EQU	0x5
TL_DOM          EQU	0x6
TL_HOS          EQU	0x7
;	REGISTER SWITCH:
XOR_AB          EQU	0x7
TIME_DISP       EQU .20
;
CSA1            EQU	0x73
CSA2            EQU	0x74
CSA3            EQU	0x75
CSB1            EQU	0x76
CSB2            EQU	0x77
CSB3            EQU	0x78
CSC1            EQU	0x79
CSC2            EQU	0x7A
CSC3            EQU	0x7B
;
	CBLOCK		0x20
	DIR_DISP            ;oznaceni pozice MX displeje
	MIN                 ;minuty
	SEC                 ;sekundy
    DESETINA            ;desetina sekundy
	SETINA
	KEYS
	MENU                ;prubeh zapasu 5-1.polocas,6-2.polocas,7-BLIK_M
	CISLO               ;promenna pro prevod
	CISLA32
	CISLA10             ;prevod desitkove
    JEDNA
	DESET
	STO
	TISIC
	DTISIC
	DISP_D1             ;display 10MIN
	DISP_D2             ;1MIN
	DISP_D3             ;10SEC
	DISP_D4             ;1SEC
	DISP_D5             ;10DOM
	DISP_D6             ;1DOM
	DISP_D7             ;10HOS
	DISP_D8             ;1HOS
    D100
	D10
	D1                  ;pro prevod na desitkovou soustavu
	TMP_L
	TIM                 ;registr casovacu
	SWITCH
	SWITCH_1
    TMP_SW
    KEYS_LONG
	TIMR1
	TIMR2L
	TIMR2H              ;casovac dlouhy
	TIMR3L
	TIMR3M
	TIMR3H
    CLOCK
	ENDC
;===================================================================================
            END			; Konec programu
;===================================================================================

